@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2010 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -91,6 +91,7 @@ MNEM_TAB mnem_flags[] = {
 	{"$alt",       20},   // use alternative pronunciation
 	{"$alt2",      21},
 	{"$combine",   22},   // Combine with the next word
+	{"$alt3",      23},
 
 	{"$dot",       24},   // ignore '.' after this word (abbreviation)
 	{"$hasdot",    25},   // use this pronunciation if there is a dot after the word
@@ -690,9 +691,14 @@ static int compile_dictlist_file(const char *path, const char* filename)
 	
 	text_mode = 0;
 
-	sprintf(fname,"%s%s",path,filename);
+	// try with and without '.txt' extension
+	sprintf(fname,"%s%s.txt",path,filename);
 	if((f_in = fopen(fname,"r")) == NULL)
-		return(-1);
+	{
+		sprintf(fname,"%s%s",path,filename);
+		if((f_in = fopen(fname,"r")) == NULL)
+			return(-1);
+	}
 
 	fprintf(f_log,"Compiling: '%s'\n",fname);
 
@@ -860,6 +866,9 @@ static void copy_rule_string(char *string, int &state)
 				case 'X':
 					c = RULE_NOVOWELS;
 					break;
+				case 'J':
+					c = RULE_SKIPCHARS;
+					break;
 				case 'L':
 					// expect two digits
 					c = *p++ - '0';
@@ -1118,8 +1127,8 @@ static char *compile_rule(char *input)
 }  //  end of compile_rule
 
 
-static int __cdecl string_sorter(char **a, char **b)
-{//=================================================
+int __cdecl string_sorter(char **a, char **b)
+{//===========================================
 	char *pa, *pb;
 	int ix;
 
@@ -1659,14 +1668,18 @@ int CompileDictionary(const char *dsource, const char *dict_name, FILE *log, cha
 	if(f_log == NULL)
 		f_log = stderr;
 
+	// try with and without '.txt' extension
 	sprintf(path,"%s%s_",dsource,dict_name);
-	sprintf(fname_in,"%srules",path);
-	f_in = fopen_log(fname_in,"r");
-	if(f_in == NULL)
+	sprintf(fname_in,"%srules.txt",path);
+	if((f_in = fopen(fname_in,"r")) == NULL)
 	{
-		if(fname_err)
-			strcpy(fname_err,fname_in);
-		return(-1);
+		sprintf(fname_in,"%srules",path);
+		if((f_in = fopen_log(fname_in,"r")) == NULL)
+		{
+			if(fname_err)
+				strcpy(fname_err,fname_in);
+			return(-1);
+		}
 	}
 
 	sprintf(fname_out,"%s%c%s_dict",path_home,PATHSEP,dict_name);
