@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2010 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -235,6 +235,7 @@ void SetIndicLetters(Translator *tr)
 	SetLetterBitsRange(tr,LETTERGP_Y,0x3e,0x4c);   // + vowel signs
 
 	tr->langopts.param[LOPT_UNPRONOUNCABLE] = 1;   // disable check for unpronouncable words
+	tr->langopts.suffix_add_e = tr->letter_bits_offset + 0x4d;   //virama
 }
 
 void SetupTranslator(Translator *tr, const short *lengths, const unsigned char *amps)
@@ -361,6 +362,7 @@ Translator *SelectTranslator(const char *name)
 			tr->langopts.stress_rule = STRESSPOSN_1L;
 			tr->langopts.numbers = NUM_HUNDRED_AND | NUM_ROMAN | NUM_1900;
 			tr->langopts.param[LOPT_COMBINE_WORDS] = 2;       // allow "mc" to cmbine with the following word
+			tr->langopts.suffix_add_e = 'e';
 		}
 		break;
 
@@ -793,7 +795,7 @@ SetLengthMods(tr,3);  // all equal
 
 	case L('n','o'):  // Norwegian
 		{
-			static const short stress_lengths_no[8] = {160,140, 200,200, 0,0, 220,210};
+			static const short stress_lengths_no[8] = {160,140, 200,200, 0,0, 220,230};
 
 			SetupTranslator(tr,stress_lengths_no,NULL);
 			tr->langopts.stress_rule = STRESSPOSN_1L;
@@ -1183,8 +1185,9 @@ typedef struct {
 */
 
 
-#define RUSSIAN2
+
 #ifdef RUSSIAN2
+// This is now done in the phoneme data, ph_russian
 
 int ChangePhonemes_ru(Translator *tr, PHONEME_LIST2 *phlist, int n_ph, int index, PHONEME_TAB *ph, CHANGEPH *ch)
 {//=============================================================================================================
@@ -1276,96 +1279,6 @@ PH('V','#'),PH('I','3'),PH('I','2'),PH('E','3')};
 
 	return(0);
 }
-#else
 
-
-int ChangePhonemes_ru(Translator *tr, PHONEME_LIST2 *phlist, int n_ph, int index, PHONEME_TAB *ph, CHANGEPH *ch)
-{//=============================================================================================================
-// Called for each phoneme in the phoneme list, to allow a language to make changes
-// flags: bit 0=1 last phoneme in a word
-//        bit 1=1 this is the highest stressed vowel in the current word
-//        bit 2=1 after the highest stressed vowel in the current word
-//        bit 3=1 the phonemes were specified explicitly, or found from an entry in the xx_list dictionary
-// ph     The current phoneme
-
-	int variant;
-	int vowelix;
-	PHONEME_TAB *prev, *next;
-
-	if(ch->flags & 8)
-		return(0);    // full phoneme translation has already been given
-
-	// Russian vowel softening and reduction rules
-	if(ph->type == phVOWEL)
-	{
-		#define N_VOWELS_RU   7
-		static unsigned char vowels_ru[N_VOWELS_RU] = {'a','A','o','E','i','u','y'};
-
-		// each line gives: soft, reduced, soft-reduced, post-tonic
-		static unsigned short vowel_replace[N_VOWELS_RU][4] = {
-			{'&', 'V', 'I', 'V'},  // a
-			{'&', 'V', 'I', 'V'},  // A
-			{'8', 'V', 'I', 'V'},  // o
-			{'e', 'I', 'I', 'I'},  // E
-			{'i', 'I', 'I', 'I'},  // i
-			{'u'+('"'<<8), 'U', 'U', 'U'},  // u
-			{'y', 'Y', 'Y', 'Y'}};  // y
-
-		prev = phoneme_tab[phlist[index-1].phcode];
-		next = phoneme_tab[phlist[index+1].phcode];
-
-if(prev->mnemonic == 'j')
-  return(0);
-
-		// lookup the vowel name to get an index into the vowel_replace[] table
-		for(vowelix=0; vowelix<N_VOWELS_RU; vowelix++)
-		{
-			if(vowels_ru[vowelix] == ph->mnemonic)
-				break;
-		}
-		if(vowelix == N_VOWELS_RU)
-			return(0);
-
-		// do we need a variant of this vowel, depending on the stress and adjacent phonemes ?
-		variant = -1;
-		if(ch->flags & 2)
-		{
-			// a stressed vowel
-			if((prev->phflags & phPALATAL) && ((next->phflags & phPALATAL) || phoneme_tab[phlist[index+2].phcode]->mnemonic == ';'))
-			{
-				// between two palatal consonants, use the soft variant
-				variant = 0;
-			}
-		}
-		else
-		{
-			// an unstressed vowel
-			if(prev->phflags & phPALATAL)
-			{
-				variant = 2;  // unstressed soft
-			}
-			else
-			if((ph->mnemonic == 'o') && ((prev->phflags & phPLACE) == phPLACE_pla))
-			{
-				variant = 2;  // unstressed soft  ([o] vowel following:  ш ж
-			}
-			else
-			if(ch->flags & 4)
-			{
-				variant = 3;  // post tonic
-			}
-			else
-			{
-				variant = 1;  // unstressed
-			}
-		}
-		if(variant >= 0)
-		{
-			phlist[index].phcode = PhonemeCode(vowel_replace[vowelix][variant]);
-		}
-	}
-
-	return(0);
-}
 #endif
 
